package org.springblade.core.oss;

import com.obs.services.ObsClient;
import com.obs.services.model.ObjectMetadata;
import com.obs.services.model.PutObjectResult;
import lombok.SneakyThrows;
import org.springblade.core.oss.model.BladeFile;
import org.springblade.core.oss.model.OssFile;
import org.springblade.core.oss.props.OssProperties;
import org.springblade.core.oss.rule.OssRule;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.List;

/**
 * Author pangqijun
 * Date 2023/10/25
 * Description
 */
public class HuaweiObsTemplate implements OssTemplate {
	private final ObsClient obsClient;
	private final OssProperties ossProperties;
	private final OssRule ossRule;

	public void makeBucket(String bucketName) {
		if (!this.bucketExists(bucketName)) {
			this.obsClient.createBucket(this.getBucketName(bucketName));
		}

	}

	public void removeBucket(String bucketName) {
		this.obsClient.deleteBucket(this.getBucketName(bucketName));
	}

	public boolean bucketExists(String bucketName) {
		return this.obsClient.headBucket(this.getBucketName(bucketName));
	}

	public void copyFile(String bucketName, String fileName, String destBucketName) {
		this.obsClient.copyObject(this.getBucketName(bucketName), fileName, this.getBucketName(destBucketName), fileName);
	}

	public void copyFile(String bucketName, String fileName, String destBucketName, String destFileName) {
		this.obsClient.copyObject(this.getBucketName(bucketName), fileName, this.getBucketName(destBucketName), destFileName);
	}

	public OssFile statFile(String fileName) {
		return this.statFile(this.ossProperties.getBucketName(), fileName);
	}

	public OssFile statFile(String bucketName, String fileName) {
		ObjectMetadata stat = this.obsClient.getObjectMetadata(this.getBucketName(bucketName), fileName);
		OssFile ossFile = new OssFile();
		ossFile.setName(fileName);
		ossFile.setLink(this.fileLink(ossFile.getName()));
		ossFile.setHash(stat.getContentMd5());
		ossFile.setLength(stat.getContentLength());
		ossFile.setPutTime(stat.getLastModified());
		ossFile.setContentType(stat.getContentType());
		return ossFile;
	}

	public String filePath(String fileName) {
		return this.getOssHost(this.getBucketName()).concat("/").concat(fileName);
	}

	public String filePath(String bucketName, String fileName) {
		return this.getOssHost(this.getBucketName(bucketName)).concat("/").concat(fileName);
	}

	public String fileLink(String fileName) {
		return this.getOssHost().concat("/").concat(fileName);
	}

	public String fileLink(String bucketName, String fileName) {
		return this.getOssHost(this.getBucketName(bucketName)).concat("/").concat(fileName);
	}

	public BladeFile putFile(MultipartFile file) {
		return this.putFile(this.ossProperties.getBucketName(), file.getOriginalFilename(), file);
	}

	public BladeFile putFile(String fileName, MultipartFile file) {
		return this.putFile(this.ossProperties.getBucketName(), fileName, file);
	}

	@SneakyThrows
	public BladeFile putFile(String bucketName, String fileName, MultipartFile file) {
		return this.putFile(bucketName, fileName, file.getInputStream());
	}

	public BladeFile putFile(String fileName, InputStream stream) {
		return this.putFile(this.ossProperties.getBucketName(), fileName, stream);
	}

	public BladeFile putFile(String bucketName, String fileName, InputStream stream) {
		return this.put(bucketName, stream, fileName, false);
	}

	public void removeFile(String fileName) {
		this.obsClient.deleteObject(this.getBucketName(), fileName);
	}

	public void removeFile(String bucketName, String fileName) {
		this.obsClient.deleteObject(this.getBucketName(bucketName), fileName);
	}

	public void removeFiles(List<String> fileNames) {
		fileNames.forEach(this::removeFile);
	}

	public void removeFiles(String bucketName, List<String> fileNames) {
		fileNames.forEach((fileName) -> {
			this.removeFile(this.getBucketName(bucketName), fileName);
		});
	}

	public BladeFile put(String bucketName, InputStream stream, String key, boolean cover) {
		try {
			this.makeBucket(bucketName);
			String originalName = key;
			key = this.getFileName(key);
			if (cover) {
				this.obsClient.putObject(this.getBucketName(bucketName), key, stream);
			} else {
				PutObjectResult response = this.obsClient.putObject(this.getBucketName(bucketName), key, stream);
				int retry = 0;

				for(byte retryCount = 5; StringUtils.isEmpty(response.getEtag()) && retry < retryCount; ++retry) {
					response = this.obsClient.putObject(this.getBucketName(bucketName), key, stream);
				}
			}

			BladeFile file = new BladeFile();
			file.setOriginalName(originalName);
			file.setName(key);
			file.setLink(this.fileLink(bucketName, key));
			return file;
		} catch (Throwable var9) {
			throw var9;
		}
	}

	private String getFileName(String originalFilename) {
		return this.ossRule.fileName(originalFilename);
	}

	private String getBucketName() {
		return this.getBucketName(this.ossProperties.getBucketName());
	}

	private String getBucketName(String bucketName) {
		return this.ossRule.bucketName(bucketName);
	}

	public String getOssHost(String bucketName) {
		String prefix = this.ossProperties.getEndpoint().contains("https://") ? "https://" : "http://";
		return prefix + this.getBucketName(bucketName) + "." + this.ossProperties.getEndpoint().replaceFirst(prefix, "");
	}

	public String getOssHost() {
		return this.getOssHost(this.ossProperties.getBucketName());
	}

	public HuaweiObsTemplate(final ObsClient obsClient, final OssProperties ossProperties, final OssRule ossRule) {
		this.obsClient = obsClient;
		this.ossProperties = ossProperties;
		this.ossRule = ossRule;
	}
}
